import ts from 'typescript';
import { DynamodbPrimitiveType } from './dynamodb_primitive_types';
export interface DynamodbItemField {
    generateCode(argName: string): ts.ObjectLiteralElementLike | undefined;
    generateCodeForUnmarshal(argName: string): ts.ObjectLiteralElementLike | undefined;
}
export declare class PrimitiveField implements DynamodbItemField {
    private readonly fieldName;
    private readonly fieldType;
    constructor(fieldName: string, fieldType: DynamodbPrimitiveType);
    generateCode(argName: string): ts.ObjectLiteralElementLike | undefined;
    generateCodeForUnmarshal(argName: string): ts.ObjectLiteralElementLike | undefined;
}
export declare class ArrayField implements DynamodbItemField {
    readonly fieldName: string;
    readonly valueType: DynamodbPrimitiveType;
    constructor(fieldName: string, valueType: DynamodbPrimitiveType);
    generateCode(argName: string): ts.ObjectLiteralElementLike | undefined;
    generateCodeForUnmarshal(argName: string): ts.ObjectLiteralElementLike | undefined;
}
export declare class SetField implements DynamodbItemField {
    readonly fieldName: string;
    readonly valueType: DynamodbPrimitiveType;
    readonly shouldLenientTypeCheck: boolean;
    constructor(fieldName: string, valueType: DynamodbPrimitiveType, shouldLenientTypeCheck: boolean);
    generateCode(argName: string): ts.ObjectLiteralElementLike | undefined;
    generateCodeForUnmarshal(argName: string): ts.ObjectLiteralElementLike | undefined;
}
export declare class MapField implements DynamodbItemField {
    readonly fieldName: string;
    readonly keyValueType: DynamodbPrimitiveType;
    readonly valueType: DynamodbPrimitiveType;
    readonly shouldLenientTypeCheck: boolean;
    constructor(fieldName: string, keyValueType: DynamodbPrimitiveType, valueType: DynamodbPrimitiveType, shouldLenientTypeCheck: boolean);
    generateCode(argName: string): ts.ObjectLiteralElementLike | undefined;
    generateCodeForUnmarshal(argName: string): ts.ObjectLiteralElementLike | undefined;
}
export declare class KeyValuePairMapField implements DynamodbItemField {
    readonly fieldName: string;
    readonly keyValueType: DynamodbPrimitiveType;
    readonly valueType: DynamodbPrimitiveType;
    readonly shouldLenientTypeCheck: boolean;
    constructor(fieldName: string, keyValueType: DynamodbPrimitiveType, valueType: DynamodbPrimitiveType, shouldLenientTypeCheck: boolean);
    generateCode(argName: string): ts.ObjectLiteralElementLike | undefined;
    generateCodeForUnmarshal(argName: string): ts.ObjectLiteralElementLike | undefined;
}
