"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KeyValuePairMapField = exports.MapField = exports.SetField = exports.ArrayField = exports.PrimitiveField = void 0;
const typescript_1 = __importDefault(require("typescript"));
const dynamodb_primitive_types_1 = require("./dynamodb_primitive_types");
const logger_1 = require("./logger");
class PrimitiveField {
    constructor(fieldName, fieldType) {
        this.fieldName = fieldName;
        this.fieldType = fieldType;
    }
    generateCode(argName) {
        const toStr = this.fieldType.kind === dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Number ? '.toString()' : '';
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createObjectLiteralExpression([
            typescript_1.default.factory.createPropertyAssignment(this.fieldType.kind, typescript_1.default.factory.createIdentifier(`${argName}.${this.fieldName}${toStr}` +
                (this.fieldType.kind === dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Null ? ' === null' : ''))),
        ], true));
    }
    generateCodeForUnmarshal(argName) {
        const valueConvertingOp = (0, dynamodb_primitive_types_1.dynamodbPrimitiveValueToJSValueConvertingOp)(this.fieldType);
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createIdentifier(`${valueConvertingOp.leftOp}${argName}['${this.fieldName}']?.${this.fieldType.kind}${valueConvertingOp.rightOp}`));
    }
}
exports.PrimitiveField = PrimitiveField;
class ArrayField {
    constructor(fieldName, valueType) {
        this.fieldName = fieldName;
        this.valueType = valueType;
    }
    generateCode(argName) {
        const toStr = this.valueType.kind === dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Number ? '.toString()' : '';
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createObjectLiteralExpression([
            typescript_1.default.factory.createPropertyAssignment('L', typescript_1.default.factory.createIdentifier(`${argName}.${this.fieldName}.map(v => new Object({${this.valueType.kind}: v${toStr}}))`)),
        ], true));
    }
    generateCodeForUnmarshal(argName) {
        const valueConvertingOp = (0, dynamodb_primitive_types_1.dynamodbPrimitiveValueToJSValueConvertingOp)(this.valueType);
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createIdentifier(`${argName}['${this.fieldName}']?.L?.map(v => ${valueConvertingOp.leftOp}v.${this.valueType.kind}${valueConvertingOp.rightOp}) || []`));
    }
}
exports.ArrayField = ArrayField;
class SetField {
    constructor(fieldName, valueType, shouldLenientTypeCheck) {
        this.fieldName = fieldName;
        this.valueType = valueType;
        this.shouldLenientTypeCheck = shouldLenientTypeCheck;
    }
    generateCode(argName) {
        if (this.valueType.kind !== dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.String &&
            this.valueType.kind !== dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Number &&
            this.valueType.kind !== dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Binary) {
            const msg = `a type parameter of the Set type property "${this.fieldName}" is unsupported one`;
            if (this.shouldLenientTypeCheck) {
                (0, logger_1.warn)(msg);
                return undefined;
            }
            throw new Error(msg);
        }
        const mapToStr = this.valueType.kind === dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Number ? '.map(v => v.toString())' : '';
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createObjectLiteralExpression([
            typescript_1.default.factory.createPropertyAssignment(this.valueType.kind + 'S', typescript_1.default.factory.createIdentifier(`Array.from(${argName}.${this.fieldName}.values())${mapToStr}`)),
        ], true));
    }
    generateCodeForUnmarshal(argName) {
        const valueConvertingOp = (0, dynamodb_primitive_types_1.dynamodbPrimitiveValueToJSValueConvertingOp)(this.valueType);
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createIdentifier(`${argName}['${this.fieldName}']?.${this.valueType.kind}S?.map(v => ${valueConvertingOp.leftOp}v${valueConvertingOp.rightOp}) || new Set()`));
    }
}
exports.SetField = SetField;
class MapField {
    constructor(fieldName, keyValueType, valueType, shouldLenientTypeCheck) {
        this.fieldName = fieldName;
        this.keyValueType = keyValueType;
        this.valueType = valueType;
        this.shouldLenientTypeCheck = shouldLenientTypeCheck;
    }
    generateCode(argName) {
        if (this.keyValueType.kind !== dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.String) {
            const msg = `a Map type property "${this.fieldName}" has non-string key type`;
            if (this.shouldLenientTypeCheck) {
                (0, logger_1.warn)(msg);
                return undefined;
            }
            throw new Error(msg);
        }
        const toStr = this.valueType.kind === dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Number ? '.toString()' : '';
        const recordIdent = typescript_1.default.factory.createIdentifier('m');
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createObjectLiteralExpression([
            typescript_1.default.factory.createPropertyAssignment('M', typescript_1.default.factory.createImmediatelyInvokedFunctionExpression([
                typescript_1.default.factory.createVariableStatement([], [typescript_1.default.factory.createVariableDeclaration(recordIdent)]),
                typescript_1.default.factory.createBinaryExpression(recordIdent, typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createIdentifier('{}')),
                typescript_1.default.factory.createForOfStatement(undefined, typescript_1.default.factory.createVariableDeclarationList([typescript_1.default.factory.createVariableDeclaration('kv')], typescript_1.default.NodeFlags.Const), typescript_1.default.factory.createIdentifier(`${argName}.${this.fieldName}`), typescript_1.default.factory.createBlock([
                    typescript_1.default.factory.createBinaryExpression(typescript_1.default.factory.createIdentifier(`${recordIdent.text}[kv[0]]`), typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createObjectLiteralExpression([
                        typescript_1.default.factory.createPropertyAssignment(this.valueType.kind, typescript_1.default.factory.createIdentifier(`kv[1]${toStr}` +
                            (this.valueType.kind === dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Null ? ' === null' : ''))),
                    ])),
                ])),
                typescript_1.default.factory.createReturnStatement(recordIdent),
            ])),
        ], true));
    }
    generateCodeForUnmarshal(argName) {
        const valueIdent = typescript_1.default.factory.createIdentifier('m');
        const recordIdent = typescript_1.default.factory.createIdentifier('r');
        const valueConvertingOp = (0, dynamodb_primitive_types_1.dynamodbPrimitiveValueToJSValueConvertingOp)(this.valueType);
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createImmediatelyInvokedFunctionExpression([
            typescript_1.default.factory.createVariableStatement([], [typescript_1.default.factory.createVariableDeclaration(valueIdent), typescript_1.default.factory.createVariableDeclaration(recordIdent)]),
            typescript_1.default.factory.createBinaryExpression(valueIdent, typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createIdentifier(`new Map()`)),
            typescript_1.default.factory.createBinaryExpression(recordIdent, typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createIdentifier(`${argName}['${this.fieldName}']?.M`)),
            typescript_1.default.factory.createForInStatement(typescript_1.default.factory.createVariableDeclarationList([typescript_1.default.factory.createVariableDeclaration('k')], typescript_1.default.NodeFlags.Const), recordIdent, typescript_1.default.factory.createBlock([
                typescript_1.default.factory.createIdentifier(`${valueIdent.text}.set(k, ${valueConvertingOp.leftOp}${recordIdent.text}[k]?.${this.valueType.kind}${valueConvertingOp.rightOp})`),
            ])),
            typescript_1.default.factory.createReturnStatement(valueIdent),
        ]));
    }
}
exports.MapField = MapField;
class KeyValuePairMapField {
    constructor(fieldName, keyValueType, valueType, shouldLenientTypeCheck) {
        this.fieldName = fieldName;
        this.keyValueType = keyValueType;
        this.valueType = valueType;
        this.shouldLenientTypeCheck = shouldLenientTypeCheck;
    }
    generateCode(argName) {
        if (this.keyValueType.kind !== dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.String) {
            const msg = `a Map type property "${this.fieldName}" has non-string key type`;
            if (this.shouldLenientTypeCheck) {
                (0, logger_1.warn)(msg);
                return undefined;
            }
            throw new Error(msg);
        }
        const toStr = this.valueType.kind === dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Number ? '.toString()' : '';
        const recordIdent = typescript_1.default.factory.createIdentifier('m');
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createObjectLiteralExpression([
            typescript_1.default.factory.createPropertyAssignment('M', typescript_1.default.factory.createImmediatelyInvokedFunctionExpression([
                typescript_1.default.factory.createVariableStatement([], [typescript_1.default.factory.createVariableDeclaration(recordIdent)]),
                typescript_1.default.factory.createBinaryExpression(recordIdent, typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createIdentifier('{}')),
                typescript_1.default.factory.createForInStatement(typescript_1.default.factory.createVariableDeclarationList([typescript_1.default.factory.createVariableDeclaration('k')], typescript_1.default.NodeFlags.Const), typescript_1.default.factory.createIdentifier(`${argName}.${this.fieldName}`), typescript_1.default.factory.createBlock([
                    typescript_1.default.factory.createBinaryExpression(typescript_1.default.factory.createIdentifier(`${recordIdent.text}[k]`), typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createObjectLiteralExpression([
                        typescript_1.default.factory.createPropertyAssignment(this.valueType.kind, typescript_1.default.factory.createIdentifier(`${argName}.${this.fieldName}[k]${toStr}` +
                            (this.valueType.kind === dynamodb_primitive_types_1.DynamodbPrimitiveTypeKinds.Null ? ' === null' : ''))),
                    ])),
                ])),
                typescript_1.default.factory.createReturnStatement(recordIdent),
            ])),
        ], true));
    }
    generateCodeForUnmarshal(argName) {
        const valueIdent = typescript_1.default.factory.createIdentifier('m');
        const recordIdent = typescript_1.default.factory.createIdentifier('r');
        const valueConvertingOp = (0, dynamodb_primitive_types_1.dynamodbPrimitiveValueToJSValueConvertingOp)(this.valueType);
        return typescript_1.default.factory.createPropertyAssignment(this.fieldName, typescript_1.default.factory.createImmediatelyInvokedFunctionExpression([
            typescript_1.default.factory.createVariableStatement([], [typescript_1.default.factory.createVariableDeclaration(valueIdent), typescript_1.default.factory.createVariableDeclaration(recordIdent)]),
            typescript_1.default.factory.createBinaryExpression(valueIdent, typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createIdentifier('{}')),
            typescript_1.default.factory.createBinaryExpression(recordIdent, typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createIdentifier(`${argName}['${this.fieldName}']?.M`)),
            typescript_1.default.factory.createForInStatement(typescript_1.default.factory.createVariableDeclarationList([typescript_1.default.factory.createVariableDeclaration('k')], typescript_1.default.NodeFlags.Const), recordIdent, typescript_1.default.factory.createBlock([
                typescript_1.default.factory.createBinaryExpression(typescript_1.default.factory.createIdentifier(`${valueIdent.text}[k]`), typescript_1.default.SyntaxKind.EqualsToken, typescript_1.default.factory.createIdentifier(`${valueConvertingOp.leftOp}${recordIdent.text}[k]?.${this.valueType.kind}${valueConvertingOp.rightOp}`)),
            ])),
            typescript_1.default.factory.createReturnStatement(valueIdent),
        ]));
    }
}
exports.KeyValuePairMapField = KeyValuePairMapField;
