"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dynamodbPrimitiveValueToJSValueConvertingOp = exports.dynamodbPrimitiveTypeFromName = exports.dynamodbPrimitiveTypeFromTypeFlag = exports.DynamodbPrimitiveTypeFlags = exports.DynamodbPrimitiveTypeKinds = exports.DynamodbPrimitiveType = void 0;
const typescript_1 = __importDefault(require("typescript"));
class DynamodbPrimitiveType {
    constructor(kind, flag) {
        this.kind = kind;
        this.flag = flag;
    }
    isBigInt() {
        return (this.kind === DynamodbPrimitiveTypeKinds.Number &&
            (this.flag & DynamodbPrimitiveTypeFlags.BigInt) === DynamodbPrimitiveTypeFlags.BigInt);
    }
}
exports.DynamodbPrimitiveType = DynamodbPrimitiveType;
var DynamodbPrimitiveTypeKinds;
(function (DynamodbPrimitiveTypeKinds) {
    DynamodbPrimitiveTypeKinds["String"] = "S";
    DynamodbPrimitiveTypeKinds["Number"] = "N";
    DynamodbPrimitiveTypeKinds["Boolean"] = "BOOL";
    DynamodbPrimitiveTypeKinds["Null"] = "NULL";
    DynamodbPrimitiveTypeKinds["Binary"] = "B";
})(DynamodbPrimitiveTypeKinds = exports.DynamodbPrimitiveTypeKinds || (exports.DynamodbPrimitiveTypeKinds = {}));
var DynamodbPrimitiveTypeFlags;
(function (DynamodbPrimitiveTypeFlags) {
    DynamodbPrimitiveTypeFlags[DynamodbPrimitiveTypeFlags["NA"] = 0] = "NA";
    DynamodbPrimitiveTypeFlags[DynamodbPrimitiveTypeFlags["BigInt"] = 1] = "BigInt";
})(DynamodbPrimitiveTypeFlags = exports.DynamodbPrimitiveTypeFlags || (exports.DynamodbPrimitiveTypeFlags = {}));
function dynamodbPrimitiveTypeFromTypeFlag(flag) {
    if (flag === undefined) {
        return undefined;
    }
    if ((flag & typescript_1.default.TypeFlags.String) === typescript_1.default.TypeFlags.String) {
        return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.String, DynamodbPrimitiveTypeFlags.NA);
    }
    if ((flag & typescript_1.default.TypeFlags.Number) === typescript_1.default.TypeFlags.Number) {
        return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.Number, DynamodbPrimitiveTypeFlags.NA);
    }
    if ((flag & typescript_1.default.TypeFlags.Boolean) === typescript_1.default.TypeFlags.Boolean) {
        return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.Boolean, DynamodbPrimitiveTypeFlags.NA);
    }
    if ((flag & typescript_1.default.TypeFlags.Unknown) === typescript_1.default.TypeFlags.Unknown) {
        return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.Null, DynamodbPrimitiveTypeFlags.NA);
    }
    return undefined;
}
exports.dynamodbPrimitiveTypeFromTypeFlag = dynamodbPrimitiveTypeFromTypeFlag;
function dynamodbPrimitiveTypeFromName(typeName) {
    if (typeName === undefined) {
        return undefined;
    }
    const unionTypes = typeName.split('|').map(t => t.trim());
    const isUndefinedType = (t) => t === 'undefined';
    typeName = unionTypes.filter(t => !isUndefinedType(t)).join('|');
    switch (typeName) {
        case 'string':
            return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.String, DynamodbPrimitiveTypeFlags.NA);
        case 'number':
            return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.Number, DynamodbPrimitiveTypeFlags.NA);
        case 'BigInt':
            return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.Number, DynamodbPrimitiveTypeFlags.BigInt);
        case 'boolean':
            return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.Boolean, DynamodbPrimitiveTypeFlags.NA);
        case 'unknown':
            return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.Null, DynamodbPrimitiveTypeFlags.NA);
        case 'Uint8Array':
            return new DynamodbPrimitiveType(DynamodbPrimitiveTypeKinds.Binary, DynamodbPrimitiveTypeFlags.NA);
        default:
            return undefined;
    }
}
exports.dynamodbPrimitiveTypeFromName = dynamodbPrimitiveTypeFromName;
function dynamodbPrimitiveValueToJSValueConvertingOp(primitiveType) {
    switch (primitiveType.kind) {
        case DynamodbPrimitiveTypeKinds.Number:
            return {
                leftOp: '(() => { const numStr = ',
                rightOp: `; return numStr === undefined ? undefined : ${primitiveType.isBigInt() ? 'BigInt' : 'Number'}(numStr); })()`,
            };
        case DynamodbPrimitiveTypeKinds.Null:
            return {
                leftOp: '(',
                rightOp: ' === true ? null : undefined)',
            };
        case DynamodbPrimitiveTypeKinds.String:
        case DynamodbPrimitiveTypeKinds.Binary:
        case DynamodbPrimitiveTypeKinds.Boolean:
        default:
            return {
                leftOp: '',
                rightOp: '',
            };
    }
}
exports.dynamodbPrimitiveValueToJSValueConvertingOp = dynamodbPrimitiveValueToJSValueConvertingOp;
