"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FromDynamodbRecordTransformer = void 0;
const typescript_1 = __importDefault(require("typescript"));
const fields_collector_1 = require("./fields_collector");
class FromDynamodbRecordTransformer {
    static visitNode(node, typeChecker) {
        var _a;
        if (node.typeArguments === undefined || node.typeArguments.length !== 1 || !node.typeArguments[0]) {
            throw new Error(`No type argument on ${FromDynamodbRecordTransformer.funcName}(). Please put a type argument on the function`);
        }
        const typeArg = node.typeArguments[0];
        const typeName = typeArg.getText();
        if (node.arguments.length < 1 || !node.arguments[0]) {
            throw new Error(`No argument on ${FromDynamodbRecordTransformer.funcName}(). Please put an argument that has ${typeName} type on the function`);
        }
        const shouldLenientTypeCheck = ((_a = node.arguments[1]) === null || _a === void 0 ? void 0 : _a.getText()) === 'true';
        const type = typeChecker.getTypeFromTypeNode(typeArg);
        if (!type.isClassOrInterface()) {
            throw new Error(`A type parameter of ${FromDynamodbRecordTransformer.funcName}() must be interface, but ${typeName} is not`);
        }
        if (type.isClass()) {
            throw new Error(`A type parameter of ${FromDynamodbRecordTransformer.funcName}() must be interface, but ${typeName} is a class`);
        }
        const argVarNameIdent = typescript_1.default.factory.createIdentifier('arg');
        const objectProps = new fields_collector_1.FieldsCollector(FromDynamodbRecordTransformer.funcName, shouldLenientTypeCheck)
            .collectFields(node, typeChecker, typeName, type)
            .map(field => {
            return field === null || field === void 0 ? void 0 : field.generateCodeForUnmarshal(argVarNameIdent.text);
        })
            .filter((c) => !!c);
        return typescript_1.default.factory.createImmediatelyInvokedFunctionExpression([typescript_1.default.factory.createReturnStatement(typescript_1.default.factory.createObjectLiteralExpression(objectProps, true))], typescript_1.default.factory.createParameterDeclaration([], undefined, argVarNameIdent, undefined, node.typeArguments[0], undefined), node.arguments[0]);
    }
}
exports.FromDynamodbRecordTransformer = FromDynamodbRecordTransformer;
FromDynamodbRecordTransformer.funcName = 'fromDynamodbRecord';
