"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const typescript_1 = __importDefault(require("typescript"));
const path_1 = __importDefault(require("path"));
const dynamodb_record_transformer_1 = require("./lib/dynamodb_record_transformer");
const from_dynamodb_record_transformer_1 = require("./lib/from_dynamodb_record_transformer");
function transformer(program) {
    return (context) => (file) => visitNodeAndChildren(file, program, context);
}
exports.default = transformer;
function visitNodeAndChildren(node, program, context) {
    return typescript_1.default.visitEachChild(visitNode(node, program), childNode => visitNodeAndChildren(childNode, program, context), context);
}
function visitNode(node, program) {
    const typeChecker = program.getTypeChecker();
    if (isDynamodbRecordImportExpression(node)) {
        return undefined;
    }
    if (isFunctionCallExpression(dynamodb_record_transformer_1.DynamodbRecordTransformer.funcName, node, typeChecker)) {
        return dynamodb_record_transformer_1.DynamodbRecordTransformer.visitNode(node, typeChecker);
    }
    if (isFunctionCallExpression(from_dynamodb_record_transformer_1.FromDynamodbRecordTransformer.funcName, node, typeChecker)) {
        return from_dynamodb_record_transformer_1.FromDynamodbRecordTransformer.visitNode(node, typeChecker);
    }
    return node;
}
const indexJs = path_1.default.join(__dirname, 'index.js');
function isDynamodbRecordImportExpression(node) {
    if (!typescript_1.default.isImportDeclaration(node)) {
        return false;
    }
    const module = node.moduleSpecifier.text;
    try {
        return (indexJs ===
            (module.startsWith('.')
                ? require.resolve(path_1.default.resolve(path_1.default.dirname(node.getSourceFile().fileName), module))
                : require.resolve(module)));
    }
    catch (e) {
        return false;
    }
}
const indexTs = path_1.default.join(__dirname, 'index.d.ts');
function isFunctionCallExpression(functionName, node, typeChecker) {
    var _a, _b;
    if (!typescript_1.default.isCallExpression(node)) {
        return false;
    }
    const declaration = (_a = typeChecker.getResolvedSignature(node)) === null || _a === void 0 ? void 0 : _a.declaration;
    if (!declaration || typescript_1.default.isJSDocSignature(declaration) || ((_b = declaration.name) === null || _b === void 0 ? void 0 : _b.getText()) !== functionName) {
        return false;
    }
    try {
        return require.resolve(declaration.getSourceFile().fileName) === indexTs;
    }
    catch (_c) {
        return false;
    }
}
